# This allows you to run timed commands like in mIRC with 
# /timer <interval> <quantity> <cmd>
# I wrote this before XiRCON supported the tcl "after" command
# so it would probably will be a better way to implement this, and I will 
# probably do so at some point in the future

set timerQueue ""

alias timer {       
    set interval [lindex [args] 0] 
    set quantity [lindex [args] 1] 
    set codeBlock [lrange [args] 2 end]
    if {![regexp {[0-9]+} $interval] || ![regexp {[0-9]+} $quantity]} {
	echo "Usage: /timer <interval> <quantity> <command>"
	return
    }
    
    echo "*** The timer (TimerID = [timer $interval $quantity $codeBlock]) \
	    will execute $quantity times at intervals of $interval seconds."
    complete
}

alias killTimer {
    set pid [lindex [args] 0]
    if {[delTimer $pid] > 0} {
	echo "*** Timer $pid removed from Queue"
    } else {
	echo "*** No Timer event found for ID $pid"
    }
    complete
}

proc delTimer {pid} {
    global timerQueue 
    set removeList "" 
    set dels 0
    foreach event $timerQueue {
	if {[lindex $event 4] == $pid} {
	    lappend removeList $event
	    set dels [expr $dels + 1]
	}
    }
    foreach event $removeList {
	removeFromList timerQueue $event
    }	
    return $dels
}	


proc timer {interval quantity codeBlock} {  
    global timerQueue
    set startTime [clock seconds]
    set timerPID [clock clicks]
    lappend timerQueue [list $startTime $interval $quantity \
	    $codeBlock $timerPID]
    return $timerPID
}

alias findtimer {
    set cmd "[args]" 
    echo "TimerID is: [findTimerID $cmd]"
    complete
}

proc findTimerID {cmd} {
    global timerQueue 
    foreach event $timerQueue {
	if {[string match "*[string tolower $cmd]*" "[string tolower \
		[lindex $event 3]]"] } {
	    return [lindex $event 4] 
	}
    }
    return -1
}


on timer {
    set removeList ""
    foreach event $timerQueue {           
	set startTime [lindex $event 0]
	set interval [lindex $event 1] 
	set quantity [lindex $event 2]
	set currTime [clock seconds]
	if {([expr ($currTime - $startTime) % $interval] == 0) && \
		($startTime != $currTime)} {
	    eval [join [lindex $event 3]]
	}
	if {$quantity > 0 && $currTime >= \
		[expr $startTime + ($interval * $quantity)]} {
	    lappend removeList $event
	}
    }
    foreach event $removeList {
	removeFromList timerQueue $event
    }		
}
